/*
 PQ Kernel 1.0
 
 This code exhibits the algorithm the computes the PQ kernel in O(n log n) time.
 
 Copyright (C) 2014  Radu Tudor Ionescu, Marius Popescu
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or any
 later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include "pqk.h"
                                   
int main(int argc, char *argv[])
{
    int i, j;
    
    printf("_________________________________________\n");
    printf("Samples: \n");
    printf("X1 = [1, 2, 3, 4]\n");
    printf("X2 = [4, 3, 2, 1]\n");
    printf("X3 = [1, 2, 4, 3]\n");
    
    double X[12] = {1, 2, 3, 4,
                    4, 3, 2, 1,
                    1, 2, 4, 3};
    
    double *K = pqk_matrix(X, 4, 3);
    
    printf("Kernel matrix : \n");
    printf("\tX1\tX2\tX3\n");
    for (i = 0; i < 3; i++)
    {
        printf("X%d\t", i + 1);
        for (j = 0; j < 3; j++)
            printf("%.1f\t", K[3 * i + j]);
    
        printf("\n");
    }
    
    printf("_________________________________________\n");
    printf("Samples: \n");
    printf("A = [1, 2, 4, 2, 1]\n");
    printf("B = [3, 1, 3, 4, 1]\n");

    double A[5] = {1, 2, 4, 2, 1};
    double B[5] = {3, 1, 3, 4, 1};

    double r = pqk(A,B,5);
    printf("Kernel function :\n");
    printf("k_PQ(A,B) = %.1f\n", r);
    
    printf("_________________________________________\n");
    printf("Samples: \n");
    printf("X1 = [1, 2, 3, 4]\n");
    printf("X2 = [4, 3, 2, 1]\n\n");
    printf("Y1 = [1, 2, 4, 3]\n");
    printf("Y2 = [2, 1, 3, 3]\n");
    printf("Y3 = [4, 1, 3, 2]\n");
    
    double Y[12] = {1, 2, 4, 3,
                    2, 1, 3, 3,
                    4, 1, 3, 2};
    
    double *Kx = pqk_cross_matrix(X, Y, 4, 2, 3);
    
    printf("Kernel matrix : \n");
    printf("\tX1\tX2\tX3\n");
    for (i = 0; i < 2; i++)
    {
        printf("Y%d\t", i + 1);
        for (j = 0; j < 3; j++)
            printf("%.1f\t", Kx[3 * i + j]);
        
        printf("\n");
    }

    return 0;
}